/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.microtesk.mmu.test.sequence.engine.memory.allocator;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import ru.ispras.fortress.util.InvariantChecks;
import ru.ispras.microtesk.settings.RegionSettings;

/**
 * {@link AddressStorage} implements a storage of addresses. 
 * 
 * @author <a href="mailto:kamkin@ispras.ru">Alexander Kamkin</a>
 */
public final class AddressStorage {
  /** Address alignment mask (11...100...0). */
  private final long addressMask;

  /** Contains region-separated aligned addresses. */
  private final Map<RegionSettings, Set<Long>> storage = new HashMap<>();

  public AddressStorage(final Collection<RegionSettings> regions, final long addressMask) {
    InvariantChecks.checkNotNull(regions);

    this.addressMask = addressMask;

    for (final RegionSettings region : regions) {
      storage.put(region, new HashSet<Long>());
    }
  }

  public void add(final long address) {
    for (final Map.Entry<RegionSettings, Set<Long>> entry : storage.entrySet()) {
      final RegionSettings region = entry.getKey();

      if (region.checkAddress(address)) {
        final Set<Long> addresses = entry.getValue();

        addresses.add(address & addressMask);
        break;
      }
    }
  }

  public Collection<Long> getAll(final RegionSettings region) {
    // Parameter {@code region} can be null;
    if (region != null) {
      return storage.get(region);
    }

    final Set<Long> result = new HashSet<>();

    for (final Set<Long> addresses : storage.values()) {
      result.addAll(addresses);
    }

    return result;
  }
}
